package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskCycleService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskPlanService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskService;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysTask;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskCycle;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskPlan;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 任务计划管理服务
 * @Reference   :
 * @Author      : yunhao
 * @Date        : 2018-12-27 10:45
 * @Modify      :
 **/
@RestController
@Api(description = "任务计划管理服务")
@RequestMapping(PlatformConstWords.BASE_URL +"/task-plan")
public class SysTaskPlanController extends CoreBaseController{

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskPlanController.class);

    @Autowired
    private ISysTaskPlanService sysTaskPlanService;

    @Autowired
    private ISysTaskCycleService sysTaskCycleService;

    @Autowired
    private ISysTaskService sysTaskService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增任务计划", notes = "任务计划")
    public ResultBean insertSysTaskTime(SysTaskPlan sysTaskPlan, Boolean isExecute) {
        try {
            // 数据校验
            ValidatorBean.beginValid(sysTaskPlan)
                    .notNull("name", sysTaskPlan.getName())
                    .notNull("groupName",sysTaskPlan.getGroupName())
                    .notNull("taskCycleId",sysTaskPlan.getTaskCycleId())
                    .notNull("taskId",sysTaskPlan.getTaskId())
                    .notNull("taskMethodType", sysTaskPlan.getTaskMethodType());

            SysTaskCycle sysTaskCycle = sysTaskCycleService.getSysTaskCycleById(sysTaskPlan.getTaskCycleId());
            ValidatorBean.checkNotNull(sysTaskCycle,"任务周期不存在");

            SysTask sysTask = sysTaskService.getSysTaskById(sysTaskPlan.getTaskId());
            ValidatorBean.checkNotNull(sysTask,"任务不存在");

            // 设置冗余信息
            sysTaskPlan.setTaskPlanExecNum(0);
            sysTaskPlan.setTaskNameRdd(sysTask.getName());
            sysTaskPlan.setTaskCycleNameRdd(sysTaskCycle.getName());
            sysTaskPlan.setTaskCycleExpsRdd(sysTaskCycle.getTaskCycleExps());
            sysTaskPlan.setTaskCycleStartDateTimeRdd(sysTaskCycle.getTaskCycleStartDatetime());
            sysTaskPlan.setTaskCycleEndDateTimeRdd(sysTaskCycle.getTaskCycleEndDatetime());
            sysTaskPlan.setTaskPlanStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            sysTaskPlan  = sysTaskPlanService.insertSysTaskPlan(sysTaskPlan,sysTaskCycle,sysTask);
            // 立即执行
            if(isExecute != null && isExecute){
                sysTaskPlanService.executeSysTaskPlan(sysTaskPlan);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysTaskPlan);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改任务计划", notes = "任务计划")
    public ResultBean updateSysTaskTime(SysTaskPlan sysTaskPlan, Boolean isExecute) {
        try {
            // 数据校验
            ValidatorBean.beginValid(sysTaskPlan)
                    .notNull("name", sysTaskPlan.getName())
                    .notNull("groupName",sysTaskPlan.getGroupName())
                    .notNull("taskCycleId",sysTaskPlan.getTaskCycleId())
                    .notNull("taskId",sysTaskPlan.getTaskId())
                    .notNull("taskMethodType", sysTaskPlan.getTaskMethodType());

            SysTaskCycle sysTaskCycle = sysTaskCycleService.getSysTaskCycleById(sysTaskPlan.getTaskCycleId());
            ValidatorBean.checkNotNull(sysTaskCycle,"任务周期不存在");

            SysTask sysTask = sysTaskService.getSysTaskById(sysTaskPlan.getTaskId());
            ValidatorBean.checkNotNull(sysTask,"任务不存在");

            sysTaskPlan.setTaskNameRdd(sysTask.getName());
            sysTaskPlan.setTaskCycleNameRdd(sysTaskCycle.getName());
            sysTaskPlan.setTaskCycleExpsRdd(sysTaskCycle.getTaskCycleExps());
            sysTaskPlan.setTaskCycleStartDateTimeRdd(sysTaskCycle.getTaskCycleStartDatetime());
            sysTaskPlan.setTaskCycleEndDateTimeRdd(sysTaskCycle.getTaskCycleEndDatetime());

            sysTaskPlanService.updateSysTaskPlan(sysTaskPlan,sysTaskCycle);
            if(isExecute != null && isExecute){
                sysTaskPlanService.executeSysTaskPlan(sysTaskPlan);
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysTaskPlan);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除任务计划", notes = "删除任务计划")
    public ResultBean deleteSysTaskTime(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");

            sysTaskPlanService.deleteSysTaskPlanById(Long.parseLong(id));
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询SysTaskTime", notes = "查询所有任务计划")
    public ResultBean findSysTaskTimeAll() {
        try {
            List<SysTaskPlan> list = sysTaskPlanService.listSysTaskPlan();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询任务计划", notes = "查询根据ID查询任务计划")
    public ResultBean getBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");

            SysTaskPlan taskTime = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(id));
            if (taskTime != null) {
                return ResultBean.success("操作成功").setResultObject(taskTime).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else {
                return ResultBean.fail("任务计划不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/status/{id}/{status}")
    @ApiOperation(value = "修改任务计划状态", notes = "根据ID修改任务计划状态")
    public ResultBean updateQuartzStatus(@PathVariable("id") String idStr,@PathVariable("status") Integer status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(idStr, "id 不能为空");
            ValidatorBean.checkNotZero(status, "状态不能为空");

            sysTaskPlanService.updateSysTaskPlanStatus(Long.parseLong(idStr),status,AuthUtil.getSessionUser());
            return new ResultBean(true);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query")
    @ApiOperation(value = "任务计划复杂查询，分页，排序",notes = "任务计划复杂查询，分页，排序")
    public ResultBean querySysTaskTimeByPager(SysTaskPlan taskTime, Pager pager){
        try {
            ListPager taskTimeListPager = sysTaskPlanService.querySysTaskPlanByPager(taskTime,pager);
            return  ResultBean.success("操作成功").setListPager(taskTimeListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/execute/{id}")
    @ApiOperation(value = "立即执行", notes = "立即执行")
    public ResultBean executeBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "任务计划id 不能为空");
            SysTaskPlan sysTaskPlan = sysTaskPlanService.getSysTaskPlanById(Long.parseLong(id));

            sysTaskPlanService.executeSysTaskPlan(sysTaskPlan);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping("/batch-delete")
    @ApiOperation(value = "根据id批量删除任务计划",notes = "根据id批量删除任务计划")
    public ResultBean deleteSysTaskPlanByIds(String[] idsStr) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(idsStr, "任务计划id 不能为空");

            sysTaskPlanService.deleteSysTaskPlanByIds(StringTool.getArrayLong(idsStr));
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}